from typing import Dict
import http.server
import socketserver
import threading
import os


class Setup:
    default_url: str = None

    def start(self):
        pass

    def stop(self):
        pass

    @staticmethod
    def parse(directory: str, args: Dict) -> "Setup":
        if args.get("type", "web") == "web":
            return Setup()

        if args["type"] == "static":
            directory = os.path.join(directory, args.get("directory", "www"))
            return StaticServer(directory, args.get("port", "8000"))


class StaticServer(Setup):
    default_url = "http://localhost:8000"
    httpd: socketserver.TCPServer = None

    def __init__(self, directory: str, port: int):
        self.directory = directory
        self.port = port

    def start(self):
        def handler(*args, **kwargs):
            http.server.SimpleHTTPRequestHandler(
                *args, directory=self.directory, **kwargs
            )

        self.httpd = socketserver.TCPServer(("", self.port), handler)
        self.thread = threading.Thread(target=self.httpd.serve_forever)
        self.thread.daemon = True
        self.thread.start()

    def stop(self):
        if self.httpd:
            self.httpd.shutdown()
            self.thread.join()



from typing import Any, List, Dict
import yaml
from lavague.tests.setup import Setup
from lavague.tests.test import ExpectTest, TaskTest


class Task:
    def __init__(
        self,
        name: str,
        url: str,
        prompt: str,
        max_steps: int,
        tests: List[TaskTest],
        user_data: Dict,
        n_attempts: int,
    ):
        self.name = name
        self.url = url
        self.prompt = prompt
        self.max_steps = max_steps
        self.tests = tests
        self.user_data = user_data
        self.n_attempts = n_attempts

    def __str__(self) -> str:
        return f"Task(name={self.name}, url={self.url}, prompt={self.prompt}, tests={len(self.tests)})"


class TestConfig:
    setup: Setup

    def __init__(self, directory: str):
        self.directory = directory
        self.tasks: list[Task] = []

        with open(f"{directory}/config.yml", "r") as f:
            self.data = yaml.safe_load(f)
            self.setup = Setup.parse(directory, self.data)

            for task_data in self.data["tasks"]:
                self.read_task(task_data)

    def read_task(self, task_data: Dict):
        prompt = task_data["prompt"]
        url = task_data.get("url", self.setup.default_url)
        user_data = {
            **self.data.get("user_data", {}),
            **task_data.get("user_data", {}),
        }
        task = Task(
            task_data.get("name", f"{prompt} from {url}"),
            url,
            prompt,
            task_data.get("max_steps", self.data.get("max_steps", 5)),
            self._parse_test(task_data),
            user_data,
            task_data.get("n_attempts", self.data.get("n_attempts", 1)),
        )
        self.tasks.append(task)

    def __str__(self) -> str:
        return "\n".join([str(t) for t in self.tasks])

    def _parse_test(self, task_data: Any) -> List[TaskTest]:
        tests: List[TaskTest] = []

        if "expect" in task_data:
            tests += ExpectTest.parse(task_data["expect"])

        return tests



from lavague.core.token_counter import TokenCounter
from llama_index.llms.openai import OpenAI
from llama_index.multi_modal_llms.openai import OpenAIMultiModal
from llama_index.embeddings.openai import OpenAIEmbedding
from lavague.core.context import Context

llm_name = "gpt-4o"
mm_llm_name = "gpt-4o"
embedding_name = "text-embedding-3-large"

# declare the token counter before any LLMs are initialized
token_counter = TokenCounter()

# init models
llm = OpenAI(model=llm_name)
mm_llm = OpenAIMultiModal(model=llm_name)
embedding = OpenAIEmbedding(model=embedding_name)

# init context
context = Context(llm, mm_llm, embedding)
